/**
 * Created : Mar 1, 2012
 */
package com.ivy.homework.phonebook;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.ivy.homework.phonebook.bean.Contact;
import com.ivy.homework.phonebook.exception.ErrorCode;
import com.ivy.homework.phonebook.exception.MyException;
import com.ivy.homework.phonebook.service.AddressBookService;
import com.ivy.homework.phonebook.service.AddressBookView;
import com.ivy.homework.phonebook.service.ProcessorService;

/**
 * Command processor of address book. this processor supports 4 kinds of commands.<br>
 * <li>create -n[name] -a[address] -p[phoneNumber]</li> all of the arguments is required<br>
 * <li>delete -n[name]</li>all of the arguments is required<br>
 * <li>set -n[name] -a[address] -p[phoneNumber]</li>all of the arguments is required<br>
 * <li>search <-n[name]>||<-a[address]>||<-p[phoneNumber]></li> fuzzy query is supported <br>
 * 
 * @author eyandai
 */
public final class Processor implements ProcessorService {
    private static final Logger LOGGER = LoggerFactory.getLogger(Processor.class);
    private AddressBookService addressBook = null;
    private AddressBookView view = null;
    private String nameOpt = "n";
    private String addressOpt = "a";
    private String phoneOpt = "p";
    private Options options = new Options();

    /**
     * Constructor of Class Processor
     * 
     * @param addressBookServiceImpl
     *            the addressBookServiceImpl hold the contacts<br>
     * @param view
     *            the view to display result<br>
     */
    public Processor(AddressBookService addressBook, AddressBookView view) {
        super();
        this.addressBook = addressBook;
        this.view = view;
        options.addOption(nameOpt, true, Constants.NAME);
        options.addOption(addressOpt, true, Constants.ADDRESS);
        options.addOption(phoneOpt, true, Constants.PHONE);
    }

    /**
     * set address book implements
     * 
     * @param view
     *            the address book service
     */
    public void setAddressBook(AddressBookService addressBook) {
        this.addressBook = addressBook;
    }

    /**
     * set display implements
     * 
     * @param view
     *            the result display service
     */
    public void setView(AddressBookView view) {
        this.view = view;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.ProcessorService#execute(java.lang.String)
     */
    public void execute(String string) {
        LOGGER.info("to execute " + string);
        String[] args = string.split(" ");
        BasicParser parser = new BasicParser();
        CommandLine cmd = null;
        try {
            cmd = parser.parse(options, args);
            String methodName = args[0].trim();
            if ("create".equals(methodName)) {
                this.create(cmd);
            } else if ("delete".equals(methodName)) {
                this.delete(cmd);
            } else if ("set".equals(methodName)) {
                this.set(cmd);
            } else if ("search".equals(methodName)) {
                this.search(cmd);
            } else {
                LOGGER.error("invalid command");
                help();
            }
        } catch (ParseException e) {
            LOGGER.error("invalid command");
            help();
        }
    }

    private void create(CommandLine cmd) {
        String name = getArgValue(cmd, nameOpt);
        String address = getArgValue(cmd, addressOpt);
        String phone = getArgValue(cmd, phoneOpt);
        try {
            validatePhoneNumber(phone);
        } catch (MyException e) {
            LOGGER.error(e.getErrorCode().getDescription());
            view.show(e.getErrorCode().getDescription());
            return;
        }
        addressBook.newContact(new Contact(name, address, phone));
        view.show("create successful");
        LOGGER.info("create successful");
    }

    private void delete(CommandLine cmd) {
        String name = getArgValue(cmd, nameOpt);
        if (name == null) {
            LOGGER.error("delete command should indicate name option");
            view.show("please use delete -n name ");
            return;
        }
        addressBook.deleteContact(name);
        view.show("delete successful");
        LOGGER.info("delete successful");
    }

    private void set(CommandLine cmd) {
        String name = getArgValue(cmd, nameOpt);
        String address = getArgValue(cmd, addressOpt);
        String phone = getArgValue(cmd, phoneOpt);
        try {
            validatePhoneNumber(phone);
        } catch (MyException e) {
            LOGGER.error(e.getErrorCode().getDescription());
            view.show(e.getErrorCode().getDescription());
            return;
        }
        addressBook.setContact(new Contact(name, address, phone));
        view.show("set successful");
        LOGGER.info("set successful");
    }

    private void search(CommandLine cmd) {
        List<Contact> list = null;
        String name = getArgValue(cmd, nameOpt);
        String address = getArgValue(cmd, addressOpt);
        String phone = getArgValue(cmd, phoneOpt);

        if (name != null) {
            list = addressBook.searchByName(name);
        } else if (address != null) {
            list = addressBook.searchByAddress(address);
        } else if (phone != null) {
            list = addressBook.searchByPhoneNumber(phone);
        }
        if (list == null) {
            list = addressBook.getAll();
        }
        view.show(list, "search successful");
        LOGGER.info("search successful");

    }

    private String getArgValue(CommandLine cmd, String Option) {
        return cmd.hasOption(Option) ? cmd.getOptionValue(Option) : null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.ProcessorService#validatePhoneNumber(java.lang.String)
     */
    public void validatePhoneNumber(String phoneNumber) throws MyException {
        String phoneRegexp = "([0-9]{2})+([0-9]{4})+([0-9]{4})+";// home-phone
        String mobileRegexp = "([0-9]{3})+([0-9]{4})+([0-9]{4})+"; // cell-phone
        if (phoneNumber == null) {
            throw new MyException(ErrorCode.PHONE_NUMBER_MISSING);
        }
        if (!(match(phoneNumber, mobileRegexp) || match(phoneNumber, phoneRegexp))) {
            throw new MyException(ErrorCode.INVALID_PHONE_NUMBER);
        }
    }

    /**
     * @param phoneNumber
     *            phone number to match
     * @param phoneRegexp
     *            regular expression used to match
     * @return true if phone number matches the regular expression
     */
    private boolean match(String phoneNumber, String phoneRegexp) {
        Pattern phonePattern = Pattern.compile(phoneRegexp);
        Matcher matcher = phonePattern.matcher(phoneNumber);
        return matcher.matches();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.ProcessorService#help()
     */
    public void help() {
        StringBuilder builder = new StringBuilder();
        builder.append("create -n[name] -a[address] -p[phoneNumber]");
        builder.append("\n");
        builder.append("delete -n[name]");
        builder.append("\n");
        builder.append("set -n[name] -a[address] -p[phoneNumber]");
        builder.append("\n");
        builder.append("search <-n[name]> <-a[address]> <-p[phoneNumber]>");
        builder.append("\n");
        view.show(builder.toString());

    }

    /**
     * stop proccess command
     */
    public void stop() {
        addressBook.close();
    }

}
